@@ -177,7 +177,7 @@ dump_object_type (GType type, const char *symbol, GOutputStream *out)
 
       parent = g_type_parent (type);
       parent_str = g_string_new ("");
-      while (parent != G_TYPE_OBJECT && parent != G_TYPE_INVALID)
+      while (parent != G_TYPE_INVALID)
         {
           if (first)
             first = FALSE;
@@ -380,8 +380,13 @@ dump_type (GType type, const char *symbol, GOutputStream *out)
  *
  * Returns: %TRUE on success, %FALSE on error
  */
+#ifndef G_IREPOSITORY_COMPILATION
+static gboolean
+dump_irepository (const char *arg, GError **error)
+#else
 gboolean
 g_irepository_dump (const char *arg, GError **error)
+#endif
 {
   GHashTable *output_types;
   char **args;
