@@ -1242,6 +1242,8 @@ serialize_type (GIrTypelibBuild    *build,
 				    node->has_length ? "," : "");
 
 	  g_string_append (str, "]");
+	  if (node->is_pointer)
+	    g_string_append (str, "*");
 	}
       else if (node->array_type == GI_ARRAY_TYPE_BYTE_ARRAY)
 	{
@@ -1476,7 +1478,7 @@ _g_ir_node_build_typelib (GIrNode         *node,
 		      ArrayTypeBlob *array = (ArrayTypeBlob *)&data[*offset2];
 		      guint32 pos;
 
-		      array->pointer = 1;
+		      array->pointer = type->is_pointer;
 		      array->reserved = 0;
 		      array->tag = type->tag;
 		      array->zero_terminated = type->zero_terminated;
@@ -1685,6 +1687,7 @@ _g_ir_node_build_typelib (GIrNode         *node,
 	blob2->may_return_null = function->result->allow_none;
 	blob2->caller_owns_return_value = function->result->transfer;
 	blob2->caller_owns_return_container = function->result->shallow_transfer;
+	blob2->skip_return = function->result->skip;
 	blob2->reserved = 0;
 	blob2->n_arguments = n;
 
@@ -1869,6 +1872,7 @@ _g_ir_node_build_typelib (GIrNode         *node,
 	blob->out = param->out;
 	blob->caller_allocates = param->caller_allocates;
 	blob->allow_none = param->allow_none;
+	blob->skip = param->skip;
 	blob->optional = param->optional;
 	blob->transfer_ownership = param->transfer;
 	blob->transfer_container_ownership = param->shallow_transfer;
