@@ -52,7 +52,7 @@ G_BEGIN_DECLS
  *
  * The typelib has the following general format.
  *
- * typelib ::= header, directory, blobs, attributes, attributedata
+ * typelib ::= header, section-index, directory, blobs, attributes, attributedata
  *
  * directory ::= list of entries
  *
@@ -233,6 +233,7 @@ typedef enum {
  * write parser which continue to work if the format is extended by
  * adding new fields before the first flexible array member in
  * variable-size blobs.
+ * @sections: Offset of section blob array
  *
  * The header structure appears exactly once at the beginning of a typelib.  It is a
  * collection of meta-information, such as the number of entries and dependencies.
@@ -278,10 +279,34 @@ typedef struct {
   guint16 interface_blob_size;
   guint16 union_blob_size;
 
+  guint32 sections;
+
   /* <private> */
-  guint16 padding[7];
+  guint16 padding[5];
 } Header;
 
+typedef enum {
+  GI_SECTION_END = 0,
+  GI_SECTION_DIRECTORY_INDEX = 1
+} SectionType;
+
+/**
+ * Section:
+ * @id: A #SectionType
+ * @offset: Integer offset for this section
+ *
+ * A section is a blob of data that's (at least theoretically) optional,
+ * and may or may not be present in the typelib.  Presently, just used
+ * for the directory index.  This allows a form of dynamic extensibility
+ * with different tradeoffs from the format minor version.
+ *
+ */
+typedef struct {
+  guint32 id;
+  guint32 offset;
+} Section;
+
+
 /**
  * DirEntry:
  * @blob_type: A #GTypelibBlobType
@@ -384,6 +409,7 @@ typedef union
  * @destroy: Index of the destroy notfication callback parameter associated with
  * the callback, or -1.
  * @arg_type: Describes the type of the parameter. See details below.
+ * @skip: Indicates that the parameter is only useful in C and should be skipped.
  *
  * Types are specified by four bytes. If the three high bytes are zero,
  * the low byte describes a basic type, otherwise the 32bit number is an
@@ -401,8 +427,9 @@ typedef struct {
   guint          transfer_container_ownership : 1;
   guint          return_value                 : 1;
   guint          scope                        : 3;
+  guint          skip                         : 1;
   /* <private> */
-  guint          reserved                     :21;
+  guint          reserved                     :20;
   /* <public> */
   gint8        closure;
   gint8        destroy;
@@ -420,6 +447,7 @@ typedef struct {
  * @caller_owns_return_container: This flag is only relevant if the return type is a container type.
  * If the flag is set, the caller is resonsible for freeing the
  * container, but not its contents.
+ * @skip_return: Indicates that the return value is only useful in C and should be skipped.
  * @n_arguments: The number of arguments that this function expects, also the length
  * of the array of ArgBlobs.
  * @arguments: An array of ArgBlob for the arguments of the function.
@@ -430,7 +458,8 @@ typedef struct {
   guint16        may_return_null              : 1;
   guint16        caller_owns_return_value     : 1;
   guint16        caller_owns_return_container : 1;
-  guint16        reserved                     :13;
+  guint16        skip_return                  : 1;
+  guint16        reserved                     :12;
 
   guint16        n_arguments;
 
@@ -632,6 +661,7 @@ typedef struct {
 /**
  * ValueBlob:
  * @deprecated: Whether this value is deprecated
+ * @unsigned_value: if set, value is a 32-bit unsigned integer cast to gint32
  * @value: The numerical value
  * @name: Name of blob
  *
@@ -639,8 +669,9 @@ typedef struct {
  */
 typedef struct {
   guint32 deprecated : 1;
+  guint32 unsigned_value : 1;
   /* <private> */
-  guint32 reserved   :31;
+  guint32 reserved   :30;
   /* <public> */
   guint32 name;
   gint32 value;
@@ -1100,6 +1131,13 @@ struct _GITypelib {
 DirEntry *g_typelib_get_dir_entry (GITypelib *typelib,
 				   guint16   index);
 
+DirEntry *g_typelib_get_dir_entry_by_name (GITypelib *typelib,
+					   const char *name);
+
+DirEntry *g_typelib_get_dir_entry_by_gtype (GITypelib *typelib,
+					    gboolean   fastpass,
+					    GType      gtype);
+
 void      g_typelib_check_sanity (void);
 
 #define   g_typelib_get_string(typelib,offset) ((const gchar*)&(typelib->data)[(offset)])
@@ -1136,6 +1174,23 @@ gboolean g_typelib_validate (GITypelib  *typelib,
 AttributeBlob *_attribute_blob_find_first (GIBaseInfo *info,
                                            guint32     blob_offset);
 
+typedef struct _GITypelibHashBuilder GITypelibHashBuilder;
+
+GITypelibHashBuilder * _gi_typelib_hash_builder_new (void);
+
+void _gi_typelib_hash_builder_add_string (GITypelibHashBuilder *builder, const char *str, guint16 value);
+
+gboolean _gi_typelib_hash_builder_prepare (GITypelibHashBuilder *builder);
+
+guint32 _gi_typelib_hash_builder_get_buffer_size (GITypelibHashBuilder *builder);
+
+void _gi_typelib_hash_builder_pack (GITypelibHashBuilder *builder, guint8* mem, guint32 size);
+
+void _gi_typelib_hash_builder_destroy (GITypelibHashBuilder *builder);
+
+guint16 _gi_typelib_hash_search (guint8* memory, const char *str);
+
+
 G_END_DECLS
 
 #endif  /* __G_TYPELIB_H__ */
